# Copyright 2014 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack.baremetal.v1 import node as _node
from openstack.baremetal.v1 import port as _port
from openstack.baremetal.v1 import port_group as _port_group
from openstack.baremetal.v1 import volume_connector as _volume_connector
from openstack.baremetal.v1 import volume_target as _volume_target

from nova import objects
from nova.virt.ironic import ironic_states


def get_test_validation(**kw):
    result = {
        'power': _node.ValidationResult(result=True, reason=None),
        'deploy': _node.ValidationResult(result=True, reason=None),
        'console': _node.ValidationResult(result=True, reason=None),
        'rescue': _node.ValidationResult(result=True, reason=None),
        'storage': _node.ValidationResult(result=True, reason=None),
    }
    result.update(kw)
    return result


def get_test_node(fields=None, **kw):
    # NOTE(stephenfin): Prevent invalid properties making their way through
    if 'uuid' in kw or 'instance_uuid' in kw or 'maintenance' in kw:
        raise Exception('Invalid property provided')

    node = {
        'id': kw.get('id', 'eeeeeeee-dddd-cccc-bbbb-aaaaaaaaaaaa'),
        'chassis_uuid': kw.get('chassis_uuid'),
        'power_state': kw.get('power_state', ironic_states.NOSTATE),
        'target_power_state': kw.get('target_power_state',
                                     ironic_states.NOSTATE),
        'provision_state': kw.get('provision_state', ironic_states.AVAILABLE),
        'target_provision_state': kw.get('target_provision_state',
                                         ironic_states.NOSTATE),
        'last_error': kw.get('last_error'),
        'instance_id': kw.get('instance_id'),
        'instance_info': kw.get('instance_info'),
        'driver': kw.get('driver', 'fake'),
        'driver_info': kw.get('driver_info', {}),
        'properties': kw.get('properties', {}),
        'reservation': kw.get('reservation'),
        'is_maintenance': kw.get('is_maintenance'),
        'network_interface': kw.get('network_interface'),
        'resource_class': kw.get('resource_class'),
        'traits': kw.get('traits', []),
        'extra': kw.get('extra', {}),
        'updated_at': kw.get('created_at'),
        'created_at': kw.get('updated_at'),
    }

    if fields is not None:
        node = {key: value for key, value in node.items() if key in fields}

    return _node.Node(**node)


def get_test_port(**kw):
    # NOTE(stephenfin): Prevent invalid properties making their way through
    if 'uuid' in kw or 'node_uuid' in kw or 'portgroup_uuid' in kw:
        raise Exception('Invalid property provided')

    port = {
        'id': kw.get('id', 'gggggggg-uuuu-qqqq-ffff-llllllllllll'),
        'node_id': kw.get('node_id', get_test_node().id),
        'address': kw.get('address', 'FF:FF:FF:FF:FF:FF'),
        'extra': kw.get('extra', {}),
        'internal_info': kw.get('internal_info', {}),
        'port_group_id': kw.get('port_group_id'),
        'created_at': kw.get('created_at'),
        'updated_at': kw.get('updated_at'),
    }

    return _port.Port(**port)


def get_test_portgroup(**kw):
    # NOTE(stephenfin): Prevent invalid properties making their way through
    if 'uuid' in kw or 'node_uuid' in kw:
        raise Exception('Invalid property provided')

    port_group = {
        'id': kw.get('id', 'deaffeed-1234-5678-9012-fedcbafedcba'),
        'node_id': kw.get('node_id', get_test_node().id),
        'address': kw.get('address', 'EE:EE:EE:EE:EE:EE'),
        'extra': kw.get('extra', {}),
        'internal_info': kw.get('internal_info', {}),
        'properties': kw.get('properties', {}),
        'mode': kw.get('mode', 'active-backup'),
        'name': kw.get('name'),
        'is_standalone_ports_supported': kw.get(
            'is_standalone_ports_supported', True,
        ),
        'created_at': kw.get('created_at'),
        'updated_at': kw.get('updated_at'),
    }

    return _port_group.PortGroup(**port_group)


def get_test_vif(**kw):
    return {
        'profile': kw.get('profile', {}),
        'ovs_interfaceid': kw.get('ovs_interfaceid'),
        'preserve_on_delete': kw.get('preserve_on_delete', False),
        'network': kw.get('network', {}),
        'devname': kw.get('devname', 'tapaaaaaaaa-00'),
        'vnic_type': kw.get('vnic_type', 'baremetal'),
        'qbh_params': kw.get('qbh_params'),
        'meta': kw.get('meta', {}),
        'details': kw.get('details', {}),
        'address': kw.get('address', 'FF:FF:FF:FF:FF:FF'),
        'active': kw.get('active', True),
        'type': kw.get('type', 'ironic'),
        'id': kw.get('id', 'aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee'),
        'qbg_params': kw.get('qbg_params'),
    }


def get_test_volume_connector(**kw):
    # NOTE(stephenfin): Prevent invalid properties making their way through
    if 'uuid' in kw or 'node_uuid' in kw:
        raise Exception('Invalid property provided')

    volume_connector = {
        'id': kw.get('id', 'hhhhhhhh-qqqq-uuuu-mmmm-bbbbbbbbbbbb'),
        'node_id': kw.get('node_id', get_test_node().id),
        'type': kw.get('type', 'iqn'),
        'connector_id': kw.get('connector_id', 'iqn.test'),
        'extra': kw.get('extra', {}),
        'created_at': kw.get('created_at'),
        'updated_at': kw.get('updated_at'),
    }

    return _volume_connector.VolumeConnector(**volume_connector)


def get_test_volume_target(**kw):
    # NOTE(stephenfin): Prevent invalid properties making their way through
    if 'uuid' in kw or 'node_uuid' in kw:
        raise Exception('Invalid property provided')

    volume_target = {
        'id': kw.get('id', 'aaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee'),
        'node_id': kw.get('node_id', get_test_node().id),
        'volume_type': kw.get('volume_type', 'iscsi'),
        'properties': kw.get('properties', {}),
        'boot_index': kw.get('boot_index', 0),
        'volume_id': kw.get(
            'volume_id', 'fffffff-gggg-hhhh-iiii-jjjjjjjjjjjj',
        ),
        'extra': kw.get('extra', {}),
        'created_at': kw.get('created_at'),
        'updated_at': kw.get('updated_at'),
    }

    return _volume_target.VolumeTarget(**volume_target)


def get_test_flavor(**kw):
    default_extra_specs = {
        'baremetal:deploy_kernel_id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
        'baremetal:deploy_ramdisk_id': 'bbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb',
    }
    flavor = {
        'name': kw.get('name', 'fake.flavor'),
        'extra_specs': kw.get('extra_specs', default_extra_specs),
        'swap': kw.get('swap', 0),
        'root_gb': 1,
        'memory_mb': 1,
        'vcpus': 1,
        'ephemeral_gb': kw.get('ephemeral_gb', 0),
    }

    return objects.Flavor(**flavor)


def get_test_image_meta(**kw):
    return objects.ImageMeta.from_dict(
        {'id': kw.get('id', 'cccccccc-cccc-cccc-cccc-cccccccccccc')},
    )



# Copyright 2015 NEC Corporation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from nova.tests.functional.api_sample_tests import test_servers


class ServerPasswordSampleJsonTests(test_servers.ServersSampleBase):
    sample_dir = "os-server-password"

    @mock.patch("nova.api.metadata.password.extract_password")
    def test_get_password(self, mock_extract_password):
        password = ("xlozO3wLCBRWAa2yDjCCVx8vwNPypxnypmRYDa/zErlQ+EzPe1S/"
                    "Gz6nfmC52mOlOSCRuUOmG7kqqgejPof6M7bOezS387zjq4LSvvwp"
                    "28zUknzy4YzfFGhnHAdai3TxUJ26pfQCYrq8UTzmKF2Bq8ioSEtV"
                    "VzM0A96pDh8W2i7BOz6MdoiVyiev/I1K2LsuipfxSJR7Wdke4zNX"
                    "JjHHP2RfYsVbZ/k9ANu+Nz4iIH8/7Cacud/pphH7EjrY6a4RZNrj"
                    "QskrhKYed0YERpotyjYk1eDtRe72GrSiXteqCM4biaQ5w3ruS+Ac"
                    "X//PXk3uJ5kC7d67fPXaVz4WaQRYMg==")

        # Mock password since there is no api to set it
        mock_extract_password.return_value = password
        uuid = self._post_server()
        response = self._do_get('servers/%s/os-server-password' % uuid)
        subs = {'encrypted_password': password.replace('+', '\\+')}
        self._verify_response('get-password-resp', subs, response, 200)

    def test_reset_password(self):
        uuid = self._post_server()
        response = self._do_delete('servers/%s/os-server-password' % uuid)
        self.assertEqual(204, response.status_code)



# Copyright 2011 Andrew Bogott for the Wikimedia Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from webob import exc

from nova.api.openstack import wsgi


class FloatingIPDNSDomainController(wsgi.Controller):
    """DNS domain controller for OpenStack API."""

    @wsgi.expected_errors(410)
    def index(self, req):
        raise exc.HTTPGone()

    @wsgi.expected_errors(410)
    def update(self, req, id, body):
        raise exc.HTTPGone()

    @wsgi.expected_errors(410)
    def delete(self, req, id):
        raise exc.HTTPGone()


class FloatingIPDNSEntryController(wsgi.Controller):
    """DNS Entry controller for OpenStack API."""

    @wsgi.expected_errors(410)
    def show(self, req, domain_id, id):
        raise exc.HTTPGone()

    @wsgi.expected_errors(410)
    def update(self, req, domain_id, id, body):
        raise exc.HTTPGone()

    @wsgi.expected_errors(410)
    def delete(self, req, domain_id, id):
        raise exc.HTTPGone()



# Copyright 2013 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from nova.api.validation import parameter_types


create = {
    'type': 'object',
    'properties': {
        'keypair': {
            'type': 'object',
            'properties': {
                'name': parameter_types.keypair_name_special_chars,
                'public_key': {'type': 'string'},
            },
            'required': ['name'],
            'additionalProperties': False,
        },
    },
    'required': ['keypair'],
    'additionalProperties': False,
}


create_v20 = copy.deepcopy(create)
create_v20['properties']['keypair']['properties']['name'] = (parameter_types.
    name_with_leading_trailing_spaces)


create_v22 = {
    'type': 'object',
    'properties': {
        'keypair': {
            'type': 'object',
            'properties': {
                'name': parameter_types.keypair_name_special_chars,
                'type': {
                    'type': 'string',
                    'enum': ['ssh', 'x509']
                },
                'public_key': {'type': 'string'},
            },
            'required': ['name'],
            'additionalProperties': False,
        },
    },
    'required': ['keypair'],
    'additionalProperties': False,
}

create_v210 = {
    'type': 'object',
    'properties': {
        'keypair': {
            'type': 'object',
            'properties': {
                'name': parameter_types.keypair_name_special_chars,
                'type': {
                    'type': 'string',
                    'enum': ['ssh', 'x509']
                },
                'public_key': {'type': 'string'},
                'user_id': {'type': 'string'},
            },
            'required': ['name'],
            'additionalProperties': False,
        },
    },
    'required': ['keypair'],
    'additionalProperties': False,
}

create_v292 = copy.deepcopy(create_v210)
create_v292['properties']['keypair']['properties']['name'] = (parameter_types.
    keypair_name_special_chars_292)
create_v292['properties']['keypair']['required'] = ['name', 'public_key']

index_query_schema_v20 = {
    'type': 'object',
    'properties': {},
    'additionalProperties': True
}

index_query_schema_v210 = {
    'type': 'object',
    'properties': {
        'user_id': parameter_types.multi_params({'type': 'string'})
    },
    'additionalProperties': True
}

index_query_schema_v235 = copy.deepcopy(index_query_schema_v210)
index_query_schema_v235['properties'].update(
    parameter_types.pagination_parameters)

show_query_schema_v20 = index_query_schema_v20
show_query_schema_v210 = index_query_schema_v210
delete_query_schema_v20 = index_query_schema_v20
delete_query_schema_v210 = index_query_schema_v210

index_query_schema_v275 = copy.deepcopy(index_query_schema_v235)
index_query_schema_v275['additionalProperties'] = False
show_query_schema_v275 = copy.deepcopy(show_query_schema_v210)
show_query_schema_v275['additionalProperties'] = False
delete_query_schema_v275 = copy.deepcopy(delete_query_schema_v210)
delete_query_schema_v275['additionalProperties'] = False



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from nova.compute import manager as compute_manager
from nova.tests.functional import integrated_helpers


class TestServerResizeReschedule(integrated_helpers.ProviderUsageBaseTestCase):
    """Regression test for bug #1741125

    During testing in the alternate host series, it was found that retries
    when resizing an instance would always fail. This turned out to be true
    even before alternate hosts for resize was introduced. Further
    investigation showed that there was a race in call to retry the resize
    and the revert of the original attempt.

    This adds a functional regression test to show the failure. A follow up
    patch with the fix will modify the test to show it passing again.
    """

    compute_driver = 'fake.SmallFakeDriver'

    def setUp(self):
        super(TestServerResizeReschedule, self).setUp()
        self.compute1 = self._start_compute(host='host1')
        self.compute2 = self._start_compute(host='host2')
        self.compute3 = self._start_compute(host='host3')
        self.compute4 = self._start_compute(host='host4')

        flavors = self.api.get_flavors()
        self.flavor1 = flavors[0]
        self.flavor2 = flavors[1]
        if self.flavor1["disk"] > self.flavor2["disk"]:
            # Make sure that flavor1 is smaller
            self.flavor1, self.flavor2 = self.flavor2, self.flavor1

    def test_resize_reschedule_uses_host_lists(self):
        """Test that when a resize attempt fails, the retry comes from the
        supplied host_list, and does not call the scheduler.
        """
        server_req = self._build_server(
            image_uuid='155d900f-4e14-4e4c-a73d-069cbf4541e6',
            flavor_id=self.flavor1['id'],
            networks='none')

        self.first_attempt = True
        created_server = self.api.post_server({'server': server_req})
        server = self._wait_for_state_change(created_server,
                'ACTIVE')

        actual_prep_resize = compute_manager.ComputeManager._prep_resize

        def fake_prep_resize(*args, **kwargs):
            if self.first_attempt:
                # Only fail the first time
                self.first_attempt = False
                raise Exception('fake_prep_resize')
            actual_prep_resize(*args, **kwargs)

        # Yes this isn't great in a functional test, but it's simple.
        self.stub_out('nova.compute.manager.ComputeManager._prep_resize',
                      fake_prep_resize)

        server_uuid = server["id"]
        data = {"resize": {"flavorRef": self.flavor2['id']}}
        self.api.post_server_action(server_uuid, data)

        server = self._wait_for_state_change(created_server,
                'VERIFY_RESIZE')
        self.assertEqual(self.flavor2['name'],
                         server['flavor']['original_name'])



# Copyright 2021, Red Hat, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from nova.tests.functional.api import client
from nova.tests.functional import integrated_helpers


class TestMigrateFromDownHost(integrated_helpers._IntegratedTestBase):
    """Regression test for bug #1938326

    Assert the behaviour of n-api when requests are made to migrate an instance
    from a disabled, forced down, down and disabled and down compute
    host.

    Bug #1938326 specifically covering the case where a request is made and
    accepted to migrate an instance from a disabled and down compute host.
    """
    microversion = 'latest'
    ADMIN_API = True

    def _setup_compute_service(self):
        # We want the service to be marked down in a reasonable time while
        # ensuring we don't accidentally mark services as down prematurely
        self.flags(report_interval=1)
        self.flags(service_down_time=6)

        # Use two compute services to make it easier to assert the call from
        # the dest to the src, we could also test this for same host resize.
        self._start_compute('src')
        self._start_compute('dest')

    def test_migrate_from_disabled_host(self):
        """Assert that migration requests for disabled hosts are allowed
        """
        # Launch an instance on src
        server = self._create_server(host='src', networks='none')

        # Mark the compute service as disabled
        source_compute_id = self.api.get_services(
            host='src', binary='nova-compute')[0]['id']
        self.api.put_service(source_compute_id, {"status": "disabled"})
        self._wait_for_service_parameter(
            'src', 'nova-compute',
            {
                'status': 'disabled',
                'state': 'up'
            }
        )

        # Assert that we can migrate and confirm from a disabled but up compute
        self._migrate_server(server)
        self._confirm_resize(server)

    def test_migrate_from_forced_down_host(self):
        """Assert that migration requests for forced down hosts are rejected
        """
        # Launch an instance on src
        server = self._create_server(host='src', networks='none')

        # Force down the compute
        source_compute_id = self.api.get_services(
            host='src', binary='nova-compute')[0]['id']
        self.api.put_service(source_compute_id, {'forced_down': 'true'})
        # NOTE(gibi): extra retries are needed as the default 10 retries with
        # 0.5 second sleep is close to the 6 seconds down timeout
        self._wait_for_service_parameter(
            'src', 'nova-compute',
            {
                'forced_down': True,
                'state': 'down',
                'status': 'enabled'
            },
            max_retries=20,
        )

        # Assert that we cannot migrate from a forced down compute
        ex = self.assertRaises(
            client.OpenStackApiException, self._migrate_server, server)
        self.assertEqual(409, ex.response.status_code)

    def test_migrate_from_down_host(self):
        """Assert that migration requests from down hosts are rejected
        """
        # Launch an instance on src
        server = self._create_server(host='src', networks='none')

        # Stop the compute service and wait until it's down
        self.computes['src'].stop()
        # NOTE(gibi): extra retries are needed as the default 10 retries with
        # 0.5 second sleep is close to the 6 seconds down timeout
        self._wait_for_service_parameter(
            'src', 'nova-compute',
            {
                'state': 'down',
                'status': 'enabled'
            },
            max_retries=20,
        )

        # Assert that requests to migrate from down computes are rejected
        ex = self.assertRaises(
            client.OpenStackApiException, self.api.post_server_action,
            server['id'], {'migrate': None})
        self.assertEqual(409, ex.response.status_code)

    def test_migrate_from_disabled_down_host(self):
        """Assert that migration requests for disabled down hosts are rejected
        """
        # Launch an instance on src
        server = self._create_server(host='src', networks='none')

        # Mark the compute service as disabled
        source_compute_id = self.api.get_services(
            host='src', binary='nova-compute')[0]['id']
        self.api.put_service(source_compute_id, {"status": "disabled"})
        self._wait_for_service_parameter(
            'src', 'nova-compute', {'status': 'disabled'})

        # Stop the compute service and wait until it's down
        self.computes['src'].stop()
        # NOTE(gibi): extra retries are needed as the default 10 retries with
        # 0.5 second sleep is close to the 6 seconds down timeout
        self._wait_for_service_parameter(
            'src', 'nova-compute', {'state': 'down'}, max_retries=20)

        ex = self.assertRaises(
            client.OpenStackApiException, self.api.post_server_action,
            server['id'], {'migrate': None})
        self.assertEqual(409, ex.response.status_code)



# Copyright 2012 Nebula, Inc.
# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import nova.conf
from nova import objects
from nova.tests.functional.api_sample_tests import test_servers
from oslo_utils.fixture import uuidsentinel
from unittest import mock

CONF = nova.conf.CONF

fake_aggregate = {
    'deleted': 0,
    'deleted_at': None,
    'created_at': None,
    'updated_at': None,
    'id': 123,
    'uuid': uuidsentinel.fake_aggregate,
    'name': 'us-west',
    'hosts': ['host01'],
    'metadetails': {'availability_zone': 'us-west'},
}


class ShelveJsonTest(test_servers.ServersSampleBase):
    # The 'os_compute_api:os-shelve:shelve_offload' policy is admin-only
    ADMIN_API = True
    sample_dir = "os-shelve"

    def setUp(self):
        super(ShelveJsonTest, self).setUp()
        # Don't offload instance, so we can test the offload call.
        CONF.set_override('shelved_offload_time', -1)

    def _test_server_action(self, uuid, template, action, subs=None):
        subs = subs or {}
        subs.update({'action': action})
        response = self._do_post('servers/%s/action' % uuid,
                                 template, subs)
        self.assertEqual(202, response.status_code)
        self.assertEqual("", response.text)

    def test_shelve(self):
        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')

    def test_shelve_offload(self):
        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        self._test_server_action(uuid, 'os-shelve-offload', 'shelveOffload')

    def test_unshelve(self):
        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        self._test_server_action(uuid, 'os-unshelve', 'unshelve')


class UnshelveJson277Test(ShelveJsonTest):
    ADMIN_API = False
    sample_dir = "os-shelve"
    microversion = '2.77'
    scenarios = [('v2_77', {'api_major_version': 'v2.1'})]

    def setUp(self):
        super(UnshelveJson277Test, self).setUp()
        # Almost all next tests require the instance to be shelve offloaded.
        # So shelve offload the instance and skip the shelve_offload_test
        # below.
        CONF.set_override('shelved_offload_time', 0)

    def test_shelve_offload(self):
        # Skip this test as the instance is already shelve offloaded.
        pass

    def test_unshelve_with_az(self):
        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        self._test_server_action(
            uuid,
            'os-unshelve-az',
            'unshelve',
            subs={"availability_zone": "us-west"}
        )


class UnshelveJson291Test(UnshelveJson277Test):
    ADMIN_API = True
    sample_dir = "os-shelve"
    microversion = '2.91'
    scenarios = [('v2_91', {'api_major_version': 'v2.1'})]

    def _test_server_action_invalid(
            self, uuid, template, action, subs=None, msg=None):
        subs = subs or {}
        subs.update({'action': action})
        response = self._do_post('servers/%s/action' % uuid,
                                 template, subs)
        self.assertEqual(400, response.status_code)
        self.assertIn(msg, response.text)

    def test_unshelve_with_non_valid_host(self):
        """Ensure an exception rise if host is invalid and
        a http 400 error
        """
        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        self._test_server_action_invalid(
            uuid, 'os-unshelve-host',
            'unshelve',
            subs={'host': 'host01'},
            msg='Compute host host01 could not be found.')

    @mock.patch('nova.objects.aggregate._get_by_host_from_db')
    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_unshelve_with_valid_host(
            self, compute_node_get_all_by_host, mock_api_get_by_host):
        """Ensure we can unshelve to a host
        """
        # Put compute in the correct az
        mock_api_get_by_host.return_value = [fake_aggregate]

        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        fake_computes = objects.ComputeNodeList(
            objects=[
                objects.ComputeNode(
                    host='host01',
                    uuid=uuidsentinel.host1,
                    hypervisor_hostname='host01')
            ]
        )
        compute_node_get_all_by_host.return_value = fake_computes

        self._test_server_action(
            uuid,
            'os-unshelve-host',
            'unshelve',
            subs={'host': 'host01'}
        )

    @mock.patch('nova.objects.aggregate._get_by_host_from_db')
    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_unshelve_with_az_and_host(
            self, compute_node_get_all_by_host, mock_api_get_by_host):
        """Ensure we can unshelve to a host and az
        """
        # Put compute in the correct az
        mock_api_get_by_host.return_value = [fake_aggregate]

        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        fake_computes = objects.ComputeNodeList(
            objects=[
                objects.ComputeNode(
                    host='host01',
                    uuid=uuidsentinel.host1,
                    hypervisor_hostname='host01')
            ]
        )
        compute_node_get_all_by_host.return_value = fake_computes

        self._test_server_action(
            uuid,
            'os-unshelve-host',
            'unshelve',
            subs={'host': 'host01', 'availability_zone': 'us-west'},
        )

    @mock.patch('nova.objects.aggregate._get_by_host_from_db')
    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_unshelve_with_unpin_az_and_host(
            self, compute_node_get_all_by_host, mock_api_get_by_host):
        """Ensure we can unshelve to a host and az
        """
        # Put compute in the correct az
        mock_api_get_by_host.return_value = [fake_aggregate]

        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        fake_computes = objects.ComputeNodeList(
            objects=[
                objects.ComputeNode(
                    host='host01',
                    uuid=uuidsentinel.host1,
                    hypervisor_hostname='host01')
            ]
        )
        compute_node_get_all_by_host.return_value = fake_computes

        self._test_server_action(
            uuid,
            'os-unshelve-host-and-unpin-az',
            'unshelve',
            subs={'host': 'host01'},
        )

    @mock.patch('nova.objects.aggregate._get_by_host_from_db')
    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_unshelve_with_unpin_az(
            self, compute_node_get_all_by_host, mock_api_get_by_host):
        """Ensure we can unpin an az
        """
        # Put compute in the correct az
        mock_api_get_by_host.return_value = [fake_aggregate]

        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        fake_computes = objects.ComputeNodeList(
            objects=[
                objects.ComputeNode(
                    host='host01',
                    uuid=uuidsentinel.host1,
                    hypervisor_hostname='host01')
            ]
        )
        compute_node_get_all_by_host.return_value = fake_computes

        self._test_server_action(
            uuid,
            'os-unshelve-unpin-az',
            'unshelve',
            subs={'host': 'host01'},
        )


class UnshelveJson291NonAdminTest(UnshelveJson291Test):
    # Use non admin api credentials.
    ADMIN_API = False
    sample_dir = "os-shelve"
    microversion = '2.91'
    scenarios = [('v2_91', {'api_major_version': 'v2.1'})]

    def _test_server_action_invalid(self, uuid, template, action, subs=None):
        subs = subs or {}
        subs.update({'action': action})
        response = self._do_post('servers/%s/action' % uuid,
                                 template, subs)
        self.assertEqual(403, response.status_code)
        self.assertIn(
            "Policy doesn\'t allow os_compute_api:os-shelve:unshelve_to_host" +
            " to be performed.", response.text)

    def _test_server_action(self, uuid, template, action, subs=None):
        subs = subs or {}
        subs.update({'action': action})
        response = self._do_post('servers/%s/action' % uuid,
                                 template, subs)
        self.assertEqual(202, response.status_code)
        self.assertEqual('', response.text)

    def test_unshelve_with_non_valid_host(self):
        """Ensure an exception rise if user is not admin.
        a http 403 error
        """
        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        self._test_server_action_invalid(
            uuid,
            'os-unshelve-host',
            'unshelve',
            subs={'host': 'host01'}
        )

    @mock.patch('nova.objects.aggregate._get_by_host_from_db')
    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_unshelve_with_unpin_az_and_host(
            self, compute_node_get_all_by_host, mock_api_get_by_host):
        # Put compute in the correct az
        mock_api_get_by_host.return_value = [fake_aggregate]

        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        fake_computes = objects.ComputeNodeList(
            objects=[
                objects.ComputeNode(
                    host='host01',
                    uuid=uuidsentinel.host1,
                    hypervisor_hostname='host01')
            ]
        )
        compute_node_get_all_by_host.return_value = fake_computes

        self._test_server_action_invalid(
            uuid,
            'os-unshelve-host-and-unpin-az',
            'unshelve',
            subs={'host': 'host01'},
        )

    @mock.patch('nova.objects.aggregate._get_by_host_from_db')
    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_unshelve_with_valid_host(
            self, compute_node_get_all_by_host, mock_api_get_by_host):
        # Put compute in the correct az
        mock_api_get_by_host.return_value = [fake_aggregate]

        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        fake_computes = objects.ComputeNodeList(
            objects=[
                objects.ComputeNode(
                    host='host01',
                    uuid=uuidsentinel.host1,
                    hypervisor_hostname='host01')
            ]
        )
        compute_node_get_all_by_host.return_value = fake_computes

        self._test_server_action_invalid(
            uuid,
            'os-unshelve-host',
            'unshelve',
            subs={'host': 'host01'}
        )

    @mock.patch('nova.objects.aggregate._get_by_host_from_db')
    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_unshelve_with_az_and_host(
            self, compute_node_get_all_by_host, mock_api_get_by_host):
        """Ensure we can unshelve to a host and az
        """
        # Put compute in the correct az
        mock_api_get_by_host.return_value = [fake_aggregate]

        uuid = self._post_server()
        self._test_server_action(uuid, 'os-shelve', 'shelve')
        fake_computes = objects.ComputeNodeList(
            objects=[
                objects.ComputeNode(
                    host='host01',
                    uuid=uuidsentinel.host1,
                    hypervisor_hostname='host01')
            ]
        )
        compute_node_get_all_by_host.return_value = fake_computes

        self._test_server_action_invalid(
            uuid,
            'os-unshelve-host',
            'unshelve',
            subs={'host': 'host01', 'availability_zone': 'us-west'},
        )



# Copyright 2013 Intel Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Resource monitor API specification.
"""

from oslo_log import log as logging
from stevedore import enabled

import nova.conf

CONF = nova.conf.CONF
LOG = logging.getLogger(__name__)


class MonitorHandler(object):

    NAMESPACES = [
        'nova.compute.monitors.cpu',
    ]

    def __init__(self, resource_tracker):
        # Dictionary keyed by the monitor type namespace. Value is the
        # first loaded monitor of that namespace or False.
        self.type_monitor_loaded = {ns: False for ns in self.NAMESPACES}

        self.monitors = []
        for ns in self.NAMESPACES:
            plugin_mgr = enabled.EnabledExtensionManager(
                    namespace=ns,
                    invoke_on_load=True,
                    check_func=self.check_enabled_monitor,
                    invoke_args=(resource_tracker,)
            )
            self.monitors += [ext.obj for ext in plugin_mgr]

    def check_enabled_monitor(self, ext):
        """Ensures that only one monitor is specified of any type."""
        # The extension does not have a namespace attribute, unfortunately,
        # but we can get the namespace by examining the first part of the
        # entry_point_target attribute, which looks like this:
        # 'nova.compute.monitors.cpu.virt_driver:Monitor'
        ept = ext.entry_point_target
        ept_parts = ept.split(':')
        namespace_parts = ept_parts[0].split('.')
        namespace = '.'.join(namespace_parts[0:-1])
        if self.type_monitor_loaded[namespace] is not False:
            LOG.warning("Excluding %(namespace)s monitor "
                        "%(monitor_name)s. Already loaded "
                        "%(loaded_monitor)s.",
                        {'namespace': namespace,
                         'monitor_name': ext.name,
                         'loaded_monitor': self.type_monitor_loaded[namespace]
                        })
            return False

        # NOTE(jaypipes): We used to only have CPU monitors, so
        # CONF.compute_monitors could contain "virt_driver" without any monitor
        # type namespace. So, to maintain backwards-compatibility with that
        # older way of specifying monitors, we first loop through any values in
        # CONF.compute_monitors and put any non-namespace'd values into the
        # 'cpu' namespace.
        cfg_monitors = ['cpu.' + cfg if '.' not in cfg else cfg
                        for cfg in CONF.compute_monitors]
        # NOTE(jaypipes): Append 'nova.compute.monitors.' to any monitor value
        # that doesn't have it to allow CONF.compute_monitors to use shortened
        # namespaces (like 'cpu.' instead of 'nova.compute.monitors.cpu.')
        cfg_monitors = ['nova.compute.monitors.' + cfg
                        if 'nova.compute.monitors.' not in cfg else cfg
                        for cfg in cfg_monitors]
        if namespace + '.' + ext.name in cfg_monitors:
            self.type_monitor_loaded[namespace] = ext.name
            return True
        # Only log something if compute_monitors is not empty.
        if CONF.compute_monitors:
            LOG.warning("Excluding %(namespace)s monitor %(monitor_name)s. "
                        "Not in the list of enabled monitors "
                        "(CONF.compute_monitors).",
                        {'namespace': namespace, 'monitor_name': ext.name})
        return False



#    Copyright 2019 Intel Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils

from nova.db.main import api as db
from nova.objects import base
from nova.objects import fields


@base.NovaObjectRegistry.register
class ResourceMetadata(base.NovaObject):
    # Version 1.0: Initial version
    VERSION = "1.0"

    # This is parent object of specific resources.
    # And it's used to be a object field of Resource,
    # that is to say Resource.metadata.

    def __eq__(self, other):
        return base.all_things_equal(self, other)

    def __ne__(self, other):
        return not (self == other)


@base.NovaObjectRegistry.register
class Resource(base.NovaObject):
    # Version 1.0: Initial version
    VERSION = "1.0"

    fields = {
        # UUID of resource provider
        'provider_uuid': fields.UUIDField(),
        # resource class of the Resource
        'resource_class': fields.ResourceClassField(),
        # identifier is used to identify resource, it is up to virt drivers
        # for mdev, it will be a UUID, for vpmem, it's backend namespace name
        'identifier': fields.StringField(),
        # metadata is used to contain virt driver specific resource info
        'metadata': fields.ObjectField('ResourceMetadata', subclasses=True),
    }

    def __eq__(self, other):
        return base.all_things_equal(self, other)

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        metadata = self.metadata if 'metadata' in self else None
        return hash((self.provider_uuid, self.resource_class,
                     self.identifier, metadata))


@base.NovaObjectRegistry.register
class ResourceList(base.ObjectListBase, base.NovaObject):
    # Version 1.0: Initial version
    VERSION = "1.0"

    fields = {
        'objects': fields.ListOfObjectsField('Resource'),
    }

    @base.remotable_classmethod
    def get_by_instance_uuid(cls, context, instance_uuid):
        db_extra = db.instance_extra_get_by_instance_uuid(
                context, instance_uuid, columns=['resources'])
        if not db_extra or db_extra['resources'] is None:
            return None

        primitive = jsonutils.loads(db_extra['resources'])
        resources = cls.obj_from_primitive(primitive)
        return resources


@base.NovaObjectRegistry.register
class LibvirtVPMEMDevice(ResourceMetadata):
    # Version 1.0: Initial version
    VERSION = "1.0"

    fields = {
        # This is configured in file, used to generate resource class name
        # CUSTOM_PMEM_NAMESPACE_$LABEL
        'label': fields.StringField(),
        # Backend pmem namespace's name
        'name': fields.StringField(),
        # Backend pmem namespace's size
        'size': fields.IntegerField(),
        # Backend device path
        'devpath': fields.StringField(),
        # Backend pmem namespace's alignment
        'align': fields.IntegerField(),
    }

    def __hash__(self):
        # Be sure all fields are set before using hash method
        return hash((self.label, self.name, self.size,
                     self.devpath, self.align))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt
from oslo_utils.fixture import uuidsentinel
from oslo_utils import uuidutils

from nova.compute import vm_states
from nova import context as nova_context
from nova import exception
from nova import objects
from nova import test
from nova.tests import fixtures as nova_fixtures
from nova.virt.libvirt import machine_type_utils


@ddt.ddt
class TestMachineTypeUtils(test.NoDBTestCase):

    def _create_test_instance_obj(
        self,
        vm_state=vm_states.STOPPED,
        mtype=None
    ):
        instance = objects.Instance(
            uuid=uuidsentinel.instance, host='fake', node='fake',
            task_state=None, flavor=objects.Flavor(),
            project_id='fake-project', user_id='fake-user',
            vm_state=vm_state, system_metadata={},
            image_ref=uuidsentinel.image_ref
        )
        if mtype:
            instance.system_metadata = {
                'image_hw_machine_type': mtype,
            }
        return instance

    @mock.patch('nova.objects.Instance.get_by_uuid')
    @mock.patch('nova.context.target_cell')
    @mock.patch('nova.objects.InstanceMapping.get_by_instance_uuid',
        new=mock.Mock(cell_mapping=mock.sentinel.cell_mapping))
    def test_get_machine_type(self, mock_target_cell, mock_get_instance):
        mock_target_cell.return_value.__enter__.return_value = (
            mock.sentinel.cell_context)
        mock_get_instance.return_value = self._create_test_instance_obj(
            mtype='pc'
        )
        self.assertEqual(
            'pc',
            machine_type_utils.get_machine_type(
                mock.sentinel.context,
                instance_uuid=uuidsentinel.instance
            )
        )
        mock_get_instance.assert_called_once_with(
            mock.sentinel.cell_context,
            uuidsentinel.instance,
            expected_attrs=['system_metadata']
        )

    @mock.patch('nova.objects.Instance.get_by_uuid')
    @mock.patch('nova.context.target_cell')
    @mock.patch('nova.objects.InstanceMapping.get_by_instance_uuid',
        new=mock.Mock(cell_mapping=mock.sentinel.cell_mapping))
    def test_get_machine_type_none_found(
        self, mock_target_cell, mock_get_instance
    ):
        mock_target_cell.return_value.__enter__.return_value = (
            mock.sentinel.cell_context)
        mock_get_instance.return_value = self._create_test_instance_obj()
        self.assertIsNone(
            machine_type_utils.get_machine_type(
                mock.sentinel.context,
                instance_uuid=uuidsentinel.instance
            )
        )
        mock_get_instance.assert_called_once_with(
            mock.sentinel.cell_context,
            uuidsentinel.instance,
            expected_attrs=['system_metadata']
        )

    @ddt.data(
        'pc',
        'q35',
        'virt',
        's390-ccw-virtio',
        'pc-i440fx-2.12',
        'pc-q35-2.12',
        'virt-2.12',
        'pc-i440fx-rhel8.2.0',
        'pc-q35-rhel8.2.0')
    def test_check_machine_type_support(self, machine_type):
        # Assert UnsupportedMachineType isn't raised for supported types
        machine_type_utils._check_machine_type_support(
            machine_type)

    @ddt.data(
        'pc-foo',
        'pc-foo-1.2',
        'bar-q35',
        'virt-foo',
        'pc-virt')
    def test_check_machine_type_support_failure(self, machine_type):
        # Assert UnsupportedMachineType is raised for unsupported types
        self.assertRaises(
            exception.UnsupportedMachineType,
            machine_type_utils._check_machine_type_support,
            machine_type
        )

    @ddt.data(
        ('pc-i440fx-2.10', 'pc-i440fx-2.11'),
        ('pc-q35-2.10', 'pc-q35-2.11'))
    def test_check_update_to_existing_type(self, machine_types):
        # Assert that exception.InvalidMachineTypeUpdate is not raised when
        # updating to the same type or between versions of the same type
        original_type, update_type = machine_types
        machine_type_utils._check_update_to_existing_type(
            original_type, update_type)

    @ddt.data(
        ('pc', 'q35'),
        ('q35', 'pc'),
        ('pc-i440fx-2.12', 'pc-q35-2.12'),
        ('pc', 'pc-i440fx-2.12'),
        ('pc-i440fx-2.12', 'pc'),
        ('pc-i440fx-2.12', 'pc-i440fx-2.11'))
    def test_check_update_to_existing_type_failure(self, machine_types):
        # Assert that exception.InvalidMachineTypeUpdate is raised when
        # updating to a different underlying machine type or between versioned
        # and aliased machine types
        existing_type, update_type = machine_types
        self.assertRaises(
            exception.InvalidMachineTypeUpdate,
            machine_type_utils._check_update_to_existing_type,
            existing_type, update_type
        )

    @ddt.data(
        vm_states.STOPPED,
        vm_states.SHELVED,
        vm_states.SHELVED_OFFLOADED)
    def test_check_vm_state(self, vm_state):
        instance = self._create_test_instance_obj(
            vm_state=vm_state
        )
        machine_type_utils._check_vm_state(instance)

    @ddt.data(
        vm_states.ACTIVE,
        vm_states.PAUSED,
        vm_states.ERROR)
    def test_check_vm_state_failure(self, vm_state):
        instance = self._create_test_instance_obj(
            vm_state=vm_state
        )
        self.assertRaises(
            exception.InstanceInvalidState,
            machine_type_utils._check_vm_state,
            instance
        )

    @mock.patch('nova.objects.instance.Instance.save')
    @mock.patch('nova.virt.libvirt.machine_type_utils._check_vm_state')
    @mock.patch('nova.objects.Instance.get_by_uuid')
    @mock.patch('nova.context.target_cell')
    @mock.patch('nova.objects.InstanceMapping.get_by_instance_uuid',
                new=mock.Mock(cell_mapping=mock.sentinel.cell_mapping))
    def test_update_noop(
        self,
        mock_target_cell,
        mock_get_instance,
        mock_check_vm_state,
        mock_instance_save
    ):
        # Assert that update_machine_type is a noop when the type is already
        # set within the instance, even if forced
        existing_type = 'pc'
        mock_target_cell.return_value.__enter__.return_value = (
            mock.sentinel.cell_context)
        mock_get_instance.return_value = self._create_test_instance_obj(
            mtype=existing_type,
        )

        self.assertEqual(
            (existing_type, existing_type),
            machine_type_utils.update_machine_type(
                mock.sentinel.context,
                instance_uuid=uuidsentinel.instance,
                machine_type=existing_type
            ),
        )
        mock_check_vm_state.assert_not_called()
        mock_instance_save.assert_not_called()

        self.assertEqual(
            (existing_type, existing_type),
            machine_type_utils.update_machine_type(
                mock.sentinel.context,
                instance_uuid=uuidsentinel.instance,
                machine_type=existing_type,
                force=True
            ),
        )
        mock_check_vm_state.assert_not_called()
        mock_instance_save.assert_not_called()

    @ddt.data(
        ('foobar', 'foobar', None),
        ('foobar-1.3', 'foobar-1.3', 'foobar-1.2'),
        ('foobar-1.2', 'foobar-1.2', 'foobar-1.3'),
        ('foobar', 'foobar', 'q35'),
        ('pc', 'pc', 'q35'))
    @mock.patch('nova.objects.instance.Instance.save')
    @mock.patch('nova.objects.Instance.get_by_uuid')
    @mock.patch('nova.context.target_cell')
    @mock.patch('nova.objects.InstanceMapping.get_by_instance_uuid',
                new=mock.Mock(cell_mapping=mock.sentinel.cell_mapping))
    def test_update_force(
        self,
        types,
        mock_target_cell,
        mock_get_instance,
        mock_instance_save
    ):
        expected_type, update_type, existing_type = types
        mock_target_cell.return_value.__enter__.return_value = (
            mock.sentinel.cell_context)
        instance = self._create_test_instance_obj(
            mtype=existing_type
        )
        mock_get_instance.return_value = instance

        returned_type = machine_type_utils.update_machine_type(
            mock.sentinel.context,
            uuidsentinel.instance,
            machine_type=update_type,
            force=True
        )

        # Assert that the instance machine type was updated and saved
        self.assertEqual((expected_type, existing_type), returned_type)
        self.assertEqual(
            expected_type,
            instance.system_metadata.get('image_hw_machine_type')
        )
        mock_instance_save.assert_called_once()

    @ddt.data(
        ('pc', 'pc', None),
        ('q35', 'q35', None),
        ('pc-1.2', 'pc-1.2', None),
        ('pc-q35-1.2', 'pc-q35-1.2', None),
        ('pc-1.2', 'pc-1.2', 'pc-1.1'),
        ('pc-i440fx-1.2', 'pc-i440fx-1.2', 'pc-i440fx-1.1'),
        ('pc-q35-1.2', 'pc-q35-1.2', 'pc-q35-1.1'),
        ('pc-q35-rhel8.2.0', 'pc-q35-rhel8.2.0', 'pc-q35-rhel8.1.0'))
    @mock.patch('nova.objects.instance.Instance.save')
    @mock.patch('nova.objects.Instance.get_by_uuid')
    @mock.patch('nova.context.target_cell')
    @mock.patch('nova.objects.InstanceMapping.get_by_instance_uuid',
                new=mock.Mock(cell_mapping=mock.sentinel.cell_mapping))
    def test_update(
        self,
        types,
        mock_target_cell,
        mock_get_instance,
        mock_instance_save
    ):
        expected_type, update_type, existing_type = types
        mock_target_cell.return_value.__enter__.return_value = (
            mock.sentinel.cell_context)
        instance = self._create_test_instance_obj(
            mtype=existing_type
        )
        mock_get_instance.return_value = instance

        returned_type = machine_type_utils.update_machine_type(
            mock.sentinel.context,
            uuidsentinel.instance,
            machine_type=update_type
        )

        # Assert that the instance machine type was updated and saved
        self.assertEqual((expected_type, existing_type), returned_type)
        self.assertEqual(
            expected_type,
            instance.system_metadata.get('image_hw_machine_type')
        )
        mock_instance_save.assert_called_once()


class TestMachineTypeUtilsListUnset(test.NoDBTestCase):

    USES_DB_SELF = True
    NUMBER_OF_CELLS = 2

    def setUp(self):
        super().setUp()
        self.useFixture(nova_fixtures.Database(database='api'))
        self.context = nova_context.get_admin_context()

    @staticmethod
    def _create_node_in_cell(ctxt, cell, hypervisor_type, nodename):
        with nova_context.target_cell(ctxt, cell) as cctxt:
            cn = objects.ComputeNode(
                context=cctxt,
                hypervisor_type=hypervisor_type,
                hypervisor_hostname=nodename,
                # The rest of these values are fakes.
                host=uuidsentinel.host,
                vcpus=4,
                memory_mb=8 * 1024,
                local_gb=40,
                vcpus_used=2,
                memory_mb_used=2 * 1024,
                local_gb_used=10,
                hypervisor_version=1,
                cpu_info='{"arch": "x86_64"}')
            cn.create()
            return cn

    @staticmethod
    def _create_instance_in_cell(
        ctxt,
        cell,
        node,
        is_deleted=False,
        hw_machine_type=None
    ):
        with nova_context.target_cell(ctxt, cell) as cctxt:
            inst = objects.Instance(
                context=cctxt,
                host=node.host,
                node=node.hypervisor_hostname,
                compute_id=node.id,
                uuid=uuidutils.generate_uuid())
            inst.create()

            if hw_machine_type:
                inst.system_metadata = {
                    'image_hw_machine_type': hw_machine_type
                }

            if is_deleted:
                inst.destroy()

            return inst

    def _setup_instances(self):
        # Setup the required cells
        self._setup_cells()

        # Track and return the created instances so the tests can assert
        instances = {}

        # Create a node in each cell
        node1_cell1 = self._create_node_in_cell(
            self.context,
            self.cell_mappings['cell1'],
            'kvm',
            uuidsentinel.node_cell1_uuid
        )
        node2_cell2 = self._create_node_in_cell(
            self.context,
            self.cell_mappings['cell2'],
            'kvm',
            uuidsentinel.node_cell2_uuid
        )

        # Create some instances with and without machine types defined in cell1
        instances['cell1_without_mtype'] = self._create_instance_in_cell(
            self.context,
            self.cell_mappings['cell1'],
            node1_cell1
        )
        instances['cell1_with_mtype'] = self._create_instance_in_cell(
            self.context,
            self.cell_mappings['cell1'],
            node1_cell1,
            hw_machine_type='pc'
        )
        instances['cell1_with_mtype_deleted'] = self._create_instance_in_cell(
            self.context,
            self.cell_mappings['cell1'],
            node1_cell1,
            hw_machine_type='pc',
            is_deleted=True
        )

        # Repeat for cell2
        instances['cell2_without_mtype'] = self._create_instance_in_cell(
            self.context,
            self.cell_mappings['cell2'],
            node2_cell2
        )
        instances['cell2_with_mtype'] = self._create_instance_in_cell(
            self.context,
            self.cell_mappings['cell2'],
            node2_cell2,
            hw_machine_type='pc'
        )
        instances['cell2_with_mtype_deleted'] = self._create_instance_in_cell(
            self.context,
            self.cell_mappings['cell2'],
            node2_cell2,
            hw_machine_type='pc',
            is_deleted=True
        )
        return instances

    def test_fresh_install_no_cell_mappings(self):
        self.assertEqual(
            [],
            machine_type_utils.get_instances_without_type(self.context)
        )

    def test_fresh_install_no_computes(self):
        self._setup_cells()
        self.assertEqual(
            [],
            machine_type_utils.get_instances_without_type(self.context)
        )

    def test_get_from_specific_cell(self):
        instances = self._setup_instances()
        # Assert that we only see the uuid for instance cell1_without_mtype
        instance_list = machine_type_utils.get_instances_without_type(
            self.context,
            cell_uuid=self.cell_mappings['cell1'].uuid
        )
        self.assertEqual(
            instances['cell1_without_mtype'].uuid,
            instance_list[0].uuid
        )

    def test_get_multi_cell(self):
        instances = self._setup_instances()
        # Assert that we see both undeleted _without_mtype instances
        instance_list = machine_type_utils.get_instances_without_type(
            self.context,
        )
        instance_uuids = [i.uuid for i in instance_list]
        self.assertIn(
            instances['cell1_without_mtype'].uuid,
            instance_uuids
        )
        self.assertIn(
            instances['cell2_without_mtype'].uuid,
            instance_uuids
        )



#    Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

import netaddr
from oslo_db import exception as db_exc
from oslo_serialization import jsonutils
from oslo_utils.fixture import uuidsentinel
from oslo_utils import timeutils
from oslo_versionedobjects import base as ovo_base
from oslo_versionedobjects import exception as ovo_exc

from nova import conf
from nova.db.main import api as db
from nova import exception
from nova import objects
from nova.objects import base
from nova.objects import compute_node
from nova.objects import hv_spec
from nova.objects import service
from nova.tests.unit import fake_pci_device_pools
from nova.tests.unit.objects import test_objects


NOW = timeutils.utcnow().replace(microsecond=0)
fake_stats = {'num_foo': '10'}
fake_stats_db_format = jsonutils.dumps(fake_stats)
# host_ip is coerced from a string to an IPAddress
# but needs to be converted to a string for the database format
fake_host_ip = '127.0.0.1'
fake_numa_topology = objects.NUMATopology(cells=[
    objects.NUMACell(
        id=0,
        cpuset=set([1, 2]),
        pcpuset=set(),
        memory=512,
        cpu_usage=0,
        memory_usage=0,
        mempages=[],
        pinned_cpus=set(),
        siblings=[set([1]), set([2])]),
    objects.NUMACell(
        id=1,
        cpuset=set([3, 4]),
        pcpuset=set(),
        memory=512,
        cpu_usage=0,
        memory_usage=0,
        mempages=[],
        pinned_cpus=set(),
        siblings=[set([3]), set([4])])])
fake_numa_topology_db_format = fake_numa_topology._to_json()
fake_supported_instances = [('x86_64', 'kvm', 'hvm')]
fake_hv_spec = hv_spec.HVSpec(arch=fake_supported_instances[0][0],
                              hv_type=fake_supported_instances[0][1],
                              vm_mode=fake_supported_instances[0][2])
fake_supported_hv_specs = [fake_hv_spec]
# for backward compatibility, each supported instance object
# is stored as a list in the database
fake_supported_hv_specs_db_format = jsonutils.dumps([fake_hv_spec.to_list()])
fake_pci = jsonutils.dumps(fake_pci_device_pools.fake_pool_list_primitive)
fake_compute_node = {
    'created_at': NOW,
    'updated_at': None,
    'deleted_at': None,
    'deleted': False,
    'id': 123,
    'uuid': uuidsentinel.fake_compute_node,
    'service_id': None,
    'host': 'fake',
    'vcpus': 4,
    'memory_mb': 4096,
    'local_gb': 1024,
    'vcpus_used': 2,
    'memory_mb_used': 2048,
    'local_gb_used': 512,
    'hypervisor_type': 'Hyper-Dan-VM-ware',
    'hypervisor_version': 1001,
    'hypervisor_hostname': 'vm.danplanet.com',
    'free_ram_mb': 1024,
    'free_disk_gb': 256,
    'current_workload': 100,
    'running_vms': 2013,
    'cpu_info': 'Schmintel i786',
    'disk_available_least': 256,
    'metrics': '',
    'stats': fake_stats_db_format,
    'host_ip': fake_host_ip,
    'numa_topology': fake_numa_topology_db_format,
    'supported_instances': fake_supported_hv_specs_db_format,
    'pci_stats': fake_pci,
    'cpu_allocation_ratio': 16.0,
    'ram_allocation_ratio': 1.5,
    'disk_allocation_ratio': 1.0,
    'mapped': 0,
    }
# FIXME(sbauza) : For compatibility checking, to be removed once we are sure
# that all computes are running latest DB version with host field in it.
fake_old_compute_node = fake_compute_node.copy()
del fake_old_compute_node['host']
# resources are passed from the virt drivers and copied into the compute_node
fake_resources = {
    'vcpus': 2,
    'memory_mb': 1024,
    'local_gb': 10,
    'cpu_info': 'fake-info',
    'vcpus_used': 1,
    'memory_mb_used': 512,
    'local_gb_used': 4,
    'numa_topology': fake_numa_topology_db_format,
    'hypervisor_type': 'fake-type',
    'hypervisor_version': 1,
    'hypervisor_hostname': 'fake-host',
    'disk_available_least': 256,
    'host_ip': fake_host_ip,
    'supported_instances': fake_supported_instances
}
fake_compute_with_resources = objects.ComputeNode(
    vcpus=fake_resources['vcpus'],
    memory_mb=fake_resources['memory_mb'],
    local_gb=fake_resources['local_gb'],
    cpu_info=fake_resources['cpu_info'],
    vcpus_used=fake_resources['vcpus_used'],
    memory_mb_used=fake_resources['memory_mb_used'],
    local_gb_used =fake_resources['local_gb_used'],
    numa_topology=fake_resources['numa_topology'],
    hypervisor_type=fake_resources['hypervisor_type'],
    hypervisor_version=fake_resources['hypervisor_version'],
    hypervisor_hostname=fake_resources['hypervisor_hostname'],
    disk_available_least=fake_resources['disk_available_least'],
    host_ip=netaddr.IPAddress(fake_resources['host_ip']),
    supported_hv_specs=fake_supported_hv_specs,
)

CONF = conf.CONF


class _TestComputeNodeObject(object):
    def supported_hv_specs_comparator(self, expected, obj_val):
        obj_val = [inst.to_list() for inst in obj_val]
        self.assertJsonEqual(expected, obj_val)

    def pci_device_pools_comparator(self, expected, obj_val):
        if obj_val is not None:
            obj_val = obj_val.obj_to_primitive()
            self.assertJsonEqual(expected, obj_val)
        else:
            self.assertEqual(expected, obj_val)

    def comparators(self):
        return {'stats': self.assertJsonEqual,
                'host_ip': self.str_comparator,
                'supported_hv_specs': self.supported_hv_specs_comparator,
                'pci_device_pools': self.pci_device_pools_comparator,
                }

    def subs(self):
        return {'supported_hv_specs': 'supported_instances',
                'pci_device_pools': 'pci_stats'}

    @mock.patch.object(db, 'compute_node_get')
    def test_get_by_id(self, get_mock):
        get_mock.return_value = fake_compute_node
        compute = compute_node.ComputeNode.get_by_id(self.context, 123)
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())
        self.assertNotIn('uuid', compute.obj_what_changed())
        get_mock.assert_called_once_with(self.context, 123)

    @mock.patch.object(compute_node.ComputeNodeList, 'get_all_by_uuids')
    def test_get_by_uuid(self, get_all_by_uuids):
        fake_node = copy.copy(fake_compute_node)
        fake_node['stats'] = None
        get_all_by_uuids.return_value = objects.ComputeNodeList(
            objects=[objects.ComputeNode(**fake_node)])
        compute = compute_node.ComputeNode.get_by_uuid(
            self.context, uuidsentinel.fake_compute_node)
        self.assertEqual(uuidsentinel.fake_compute_node, compute.uuid)
        get_all_by_uuids.assert_called_once_with(
            self.context, [uuidsentinel.fake_compute_node])

    @mock.patch.object(compute_node.ComputeNodeList, 'get_all_by_uuids')
    def test_get_by_uuid_not_found(self, get_all_by_uuids):
        get_all_by_uuids.return_value = objects.ComputeNodeList()
        self.assertRaises(exception.ComputeHostNotFound,
                          compute_node.ComputeNode.get_by_uuid,
                          self.context, uuidsentinel.fake_compute_node)
        get_all_by_uuids.assert_called_once_with(
            self.context, [uuidsentinel.fake_compute_node])

    @mock.patch.object(db, 'compute_node_get')
    def test_get_without_mapped(self, get_mock):
        fake_node = copy.copy(fake_compute_node)
        fake_node['mapped'] = None
        get_mock.return_value = fake_node
        compute = compute_node.ComputeNode.get_by_id(self.context, 123)
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())
        self.assertIn('mapped', compute)
        self.assertEqual(0, compute.mapped)

    @mock.patch.object(objects.Service, 'get_by_id')
    @mock.patch.object(db, 'compute_node_get')
    def test_get_by_id_with_host_field_not_in_db(self, mock_cn_get,
                                                 mock_obj_svc_get):
        fake_compute_node_with_svc_id = fake_compute_node.copy()
        fake_compute_node_with_svc_id['service_id'] = 123
        fake_compute_node_with_no_host = fake_compute_node_with_svc_id.copy()
        host = fake_compute_node_with_no_host.pop('host')
        fake_service = service.Service(id=123)
        fake_service.host = host

        mock_cn_get.return_value = fake_compute_node_with_no_host
        mock_obj_svc_get.return_value = fake_service

        compute = compute_node.ComputeNode.get_by_id(self.context, 123)
        self.compare_obj(compute, fake_compute_node_with_svc_id,
                         subs=self.subs(),
                         comparators=self.comparators())

    @mock.patch.object(db, 'compute_nodes_get_by_service_id')
    def test_get_by_service_id(self, get_mock):
        get_mock.return_value = [fake_compute_node]
        compute = compute_node.ComputeNode.get_by_service_id(self.context, 456)
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())
        get_mock.assert_called_once_with(self.context, 456)

    @mock.patch.object(db, 'compute_node_get_by_host_and_nodename')
    def test_get_by_host_and_nodename(self, cn_get_by_h_and_n):
        cn_get_by_h_and_n.return_value = fake_compute_node

        compute = compute_node.ComputeNode.get_by_host_and_nodename(
            self.context, 'fake', 'vm.danplanet.com')
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())

    @mock.patch.object(db, 'compute_node_get_by_nodename')
    def test_get_by_nodename(self, cn_get_by_n):
        cn_get_by_n.return_value = fake_compute_node

        compute = compute_node.ComputeNode.get_by_nodename(
            self.context, 'vm.danplanet.com')
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())

    @mock.patch('nova.db.main.api.compute_node_get_all_by_host')
    def test_get_first_node_by_host_for_old_compat(
            self, cn_get_all_by_host):
        another_node = fake_compute_node.copy()
        another_node['hypervisor_hostname'] = 'neverland'
        cn_get_all_by_host.return_value = [fake_compute_node, another_node]

        compute = (
            compute_node.ComputeNode.get_first_node_by_host_for_old_compat(
                self.context, 'fake')
        )
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())

    @mock.patch('nova.objects.ComputeNodeList.get_all_by_host')
    def test_get_first_node_by_host_for_old_compat_not_found(
            self, cn_get_all_by_host):
        cn_get_all_by_host.side_effect = exception.ComputeHostNotFound(
            host='fake')

        self.assertRaises(
            exception.ComputeHostNotFound,
            compute_node.ComputeNode.get_first_node_by_host_for_old_compat,
            self.context, 'fake')

    @mock.patch.object(db, 'compute_node_create')
    @mock.patch(
        'nova.db.main.api.compute_node_get', return_value=fake_compute_node)
    def test_create(self, mock_get, mock_create):
        mock_create.return_value = fake_compute_node
        compute = compute_node.ComputeNode(context=self.context)
        compute.service_id = 456
        compute.uuid = uuidsentinel.fake_compute_node
        compute.stats = fake_stats
        # NOTE (pmurray): host_ip is coerced to an IPAddress
        compute.host_ip = fake_host_ip
        compute.supported_hv_specs = fake_supported_hv_specs
        with mock.patch('oslo_utils.uuidutils.generate_uuid') as mock_gu:
            compute.create()
            self.assertFalse(mock_gu.called)
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())
        param_dict = {
            'service_id': 456,
            'stats': fake_stats_db_format,
            'host_ip': fake_host_ip,
            'supported_instances': fake_supported_hv_specs_db_format,
            'uuid': uuidsentinel.fake_compute_node
        }
        mock_create.assert_called_once_with(self.context, param_dict)

    @mock.patch('nova.db.main.api.compute_node_create')
    @mock.patch('oslo_utils.uuidutils.generate_uuid')
    @mock.patch(
        'nova.db.main.api.compute_node_get', return_value=fake_compute_node)
    def test_create_allocates_uuid(self, mock_get, mock_gu, mock_create):
        mock_create.return_value = fake_compute_node
        mock_gu.return_value = fake_compute_node['uuid']
        obj = objects.ComputeNode(context=self.context)
        obj.create()
        mock_gu.assert_called_once_with()
        mock_create.assert_called_once_with(
            self.context, {'uuid': fake_compute_node['uuid']})

    @mock.patch('nova.db.main.api.compute_node_create')
    @mock.patch(
        'nova.db.main.api.compute_node_get', return_value=fake_compute_node)
    def test_recreate_fails(self, mock_get, mock_create):
        mock_create.return_value = fake_compute_node
        compute = compute_node.ComputeNode(context=self.context)
        compute.service_id = 456
        compute.uuid = uuidsentinel.fake_compute_node
        compute.create()
        self.assertRaises(exception.ObjectActionError, compute.create)
        param_dict = {'service_id': 456,
                      'uuid': uuidsentinel.fake_compute_node}
        mock_create.assert_called_once_with(self.context, param_dict)

    @mock.patch('nova.db.main.api.compute_node_create')
    def test_create_duplicate(self, mock_create):
        mock_create.side_effect = db_exc.DBDuplicateEntry
        compute = compute_node.ComputeNode(context=self.context)
        compute.service_id = 456
        compute.hypervisor_hostname = 'node1'
        self.assertRaises(exception.DuplicateRecord, compute.create)

    @mock.patch.object(db, 'compute_node_update')
    @mock.patch(
        'nova.db.main.api.compute_node_get', return_value=fake_compute_node)
    def test_save(self, mock_get, mock_update):
        mock_update.return_value = fake_compute_node
        compute = compute_node.ComputeNode(context=self.context)
        compute.id = 123
        compute.vcpus_used = 3
        compute.stats = fake_stats
        compute.uuid = uuidsentinel.fake_compute_node
        # NOTE (pmurray): host_ip is coerced to an IPAddress
        compute.host_ip = fake_host_ip
        compute.supported_hv_specs = fake_supported_hv_specs
        compute.save()
        self.compare_obj(compute, fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())
        param_dict = {
            'vcpus_used': 3,
            'stats': fake_stats_db_format,
            'host_ip': fake_host_ip,
            'supported_instances': fake_supported_hv_specs_db_format,
            'uuid': uuidsentinel.fake_compute_node,
        }
        mock_update.assert_called_once_with(self.context, 123, param_dict)

    @mock.patch('nova.db.main.api.compute_node_update')
    def test_save_pci_device_pools_empty(self, mock_update):
        fake_pci = jsonutils.dumps(
            objects.PciDevicePoolList(objects=[]).obj_to_primitive())
        compute_dict = fake_compute_node.copy()
        compute_dict['pci_stats'] = fake_pci
        mock_update.return_value = compute_dict

        compute = compute_node.ComputeNode(context=self.context)
        compute.id = 123
        compute.pci_device_pools = objects.PciDevicePoolList(objects=[])
        compute.save()
        self.compare_obj(compute, compute_dict,
                         subs=self.subs(),
                         comparators=self.comparators())

        mock_update.assert_called_once_with(
            self.context, 123, {'pci_stats': fake_pci})

    @mock.patch('nova.db.main.api.compute_node_update')
    def test_save_pci_device_pools_null(self, mock_update):
        compute_dict = fake_compute_node.copy()
        compute_dict['pci_stats'] = None
        mock_update.return_value = compute_dict

        compute = compute_node.ComputeNode(context=self.context)
        compute.id = 123
        compute.pci_device_pools = None
        compute.save()
        self.compare_obj(compute, compute_dict,
                         subs=self.subs(),
                         comparators=self.comparators())

        mock_update.assert_called_once_with(
            self.context, 123, {'pci_stats': None})

    @mock.patch.object(db, 'compute_node_create',
                       return_value=fake_compute_node)
    @mock.patch.object(db, 'compute_node_get',
                       return_value=fake_compute_node)
    def test_set_id_failure(self, mock_get, db_mock):
        compute = compute_node.ComputeNode(context=self.context,
                                           uuid=fake_compute_node['uuid'])
        compute.create()
        self.assertRaises(ovo_exc.ReadOnlyFieldError, setattr,
                          compute, 'id', 124)

    @mock.patch.object(db, 'compute_node_delete')
    def test_destroy(self, mock_delete):
        compute = compute_node.ComputeNode(context=self.context)
        compute.id = 123
        compute.destroy()
        mock_delete.assert_called_once_with(self.context, 123, constraint=None)

    def test_destroy_host_constraint(self):
        # Create compute node with host='fake'
        compute = fake_compute_with_resources.obj_clone()
        compute._context = self.context
        compute.host = 'fake'
        compute.create()
        # Simulate a compute node ownership change due to a node rebalance
        compute.host = 'different'
        self.assertRaises(exception.ObjectActionError, compute.destroy)

    @mock.patch.object(db, 'compute_node_get_all')
    def test_get_all(self, mock_get_all):
        mock_get_all.return_value = [fake_compute_node]
        computes = compute_node.ComputeNodeList.get_all(self.context)
        self.assertEqual(1, len(computes))
        self.compare_obj(computes[0], fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())
        mock_get_all.assert_called_once_with(self.context)

    @mock.patch.object(db, 'compute_node_search_by_hypervisor')
    def test_get_by_hypervisor(self, mock_search):
        mock_search.return_value = [fake_compute_node]
        computes = compute_node.ComputeNodeList.get_by_hypervisor(self.context,
                                                                  'hyper')
        self.assertEqual(1, len(computes))
        self.compare_obj(computes[0], fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())
        mock_search.assert_called_once_with(self.context, 'hyper')

    @mock.patch('nova.db.main.api.compute_node_get_all_by_pagination',
                return_value=[fake_compute_node])
    def test_get_by_pagination(self, fake_get_by_pagination):
        computes = compute_node.ComputeNodeList.get_by_pagination(
            self.context, limit=1, marker=1)
        self.assertEqual(1, len(computes))
        self.compare_obj(computes[0], fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())

    @mock.patch('nova.db.main.api.compute_nodes_get_by_service_id')
    def test__get_by_service(self, cn_get_by_svc_id):
        cn_get_by_svc_id.return_value = [fake_compute_node]
        computes = compute_node.ComputeNodeList._get_by_service(self.context,
                                                                123)
        self.assertEqual(1, len(computes))
        self.compare_obj(computes[0], fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())

    @mock.patch('nova.db.main.api.compute_node_get_all_by_host')
    def test_get_all_by_host(self, cn_get_all_by_host):
        cn_get_all_by_host.return_value = [fake_compute_node]
        computes = compute_node.ComputeNodeList.get_all_by_host(self.context,
                                                                'fake')
        self.assertEqual(1, len(computes))
        self.compare_obj(computes[0], fake_compute_node,
                         subs=self.subs(),
                         comparators=self.comparators())

    def test_compat_numa_topology(self):
        compute = compute_node.ComputeNode(numa_topology='fake-numa-topology')
        versions = ovo_base.obj_tree_get_versions('ComputeNode')
        primitive = compute.obj_to_primitive(target_version='1.4',
                                             version_manifest=versions)
        self.assertNotIn('numa_topology', primitive['nova_object.data'])

        primitive = compute.obj_to_primitive(target_version='1.5',
                                             version_manifest=versions)
        self.assertIn('numa_topology', primitive['nova_object.data'])

    def test_compat_supported_hv_specs(self):
        compute = compute_node.ComputeNode()
        compute.supported_hv_specs = fake_supported_hv_specs
        versions = ovo_base.obj_tree_get_versions('ComputeNode')
        primitive = compute.obj_to_primitive(target_version='1.5',
                                             version_manifest=versions)
        self.assertNotIn('supported_hv_specs', primitive['nova_object.data'])

        primitive = compute.obj_to_primitive(target_version='1.6',
                                             version_manifest=versions)
        self.assertIn('supported_hv_specs', primitive['nova_object.data'])

    @mock.patch('nova.objects.service.Service.get_by_compute_host')
    def test_compat_host(self, mock_get_compute):
        compute = compute_node.ComputeNode(host='fake-host')
        primitive = compute.obj_to_primitive(target_version='1.6')
        self.assertNotIn('host', primitive['nova_object.data'])

        primitive = compute.obj_to_primitive(target_version='1.7')
        self.assertIn('host', primitive['nova_object.data'])

    def test_compat_pci_device_pools(self):
        compute = compute_node.ComputeNode()
        compute.pci_device_pools = fake_pci_device_pools.fake_pool_list
        versions = ovo_base.obj_tree_get_versions('ComputeNode')
        primitive = compute.obj_to_primitive(target_version='1.8',
                                             version_manifest=versions)
        self.assertNotIn('pci_device_pools', primitive['nova_object.data'])

        primitive = compute.obj_to_primitive(target_version='1.9',
                                             version_manifest=versions)
        self.assertIn('pci_device_pools', primitive['nova_object.data'])

    @mock.patch('nova.objects.Service.get_by_compute_host')
    def test_compat_service_id(self, mock_get):
        mock_get.return_value = objects.Service(id=1)
        compute = objects.ComputeNode(host='fake-host', service_id=None)
        primitive = compute.obj_to_primitive(target_version='1.12')
        self.assertEqual(1, primitive['nova_object.data']['service_id'])

    @mock.patch('nova.objects.Service.get_by_compute_host')
    def test_compat_service_id_compute_host_not_found(self, mock_get):
        mock_get.side_effect = exception.ComputeHostNotFound(host='fake-host')
        compute = objects.ComputeNode(host='fake-host', service_id=None)
        primitive = compute.obj_to_primitive(target_version='1.12')
        self.assertEqual(-1, primitive['nova_object.data']['service_id'])

    def test_update_from_virt_driver(self):
        # copy in case the update has a side effect
        resources = copy.deepcopy(fake_resources)
        # Emulate the ironic driver which adds a uuid field.
        resources['uuid'] = uuidsentinel.node_uuid
        compute = compute_node.ComputeNode()
        compute.update_from_virt_driver(resources)
        expected = fake_compute_with_resources.obj_clone()
        expected.uuid = uuidsentinel.node_uuid
        self.assertTrue(base.obj_equal_prims(expected, compute))

    def test_update_from_virt_driver_uuid_already_set(self):
        """Tests update_from_virt_driver where the compute node object already
        has a uuid value so an error is raised.
        """
        # copy in case the update has a side effect
        resources = copy.deepcopy(fake_resources)
        # Emulate the ironic driver which adds a uuid field.
        resources['uuid'] = uuidsentinel.node_uuid
        compute = compute_node.ComputeNode(uuid=uuidsentinel.something_else)
        self.assertRaises(exception.InvalidNodeConfiguration,
            compute.update_from_virt_driver, resources)

    def test_update_from_virt_driver_missing_field(self):
        # NOTE(pmurray): update_from_virt_driver does not require
        # all fields to be present in resources. Validation of the
        # resources data structure would be done in a different method.
        resources = copy.deepcopy(fake_resources)
        del resources['vcpus']
        compute = compute_node.ComputeNode()
        compute.update_from_virt_driver(resources)
        expected = fake_compute_with_resources.obj_clone()
        del expected.vcpus
        self.assertTrue(base.obj_equal_prims(expected, compute))

    def test_update_from_virt_driver_extra_field(self):
        # copy in case the update has a side effect
        resources = copy.deepcopy(fake_resources)
        resources['extra_field'] = 'nonsense'
        compute = compute_node.ComputeNode()
        compute.update_from_virt_driver(resources)
        expected = fake_compute_with_resources
        self.assertTrue(base.obj_equal_prims(expected, compute))

    def test_update_from_virt_driver_bad_value(self):
        # copy in case the update has a side effect
        resources = copy.deepcopy(fake_resources)
        resources['vcpus'] = 'nonsense'
        compute = compute_node.ComputeNode()
        self.assertRaises(ValueError,
                          compute.update_from_virt_driver, resources)

    def test_compat_allocation_ratios(self):
        compute = compute_node.ComputeNode(
            cpu_allocation_ratio=1.0, ram_allocation_ratio=1.0)
        primitive = compute.obj_to_primitive(target_version='1.13')
        self.assertNotIn('cpu_allocation_ratio', primitive['nova_object.data'])
        self.assertNotIn('ram_allocation_ratio', primitive['nova_object.data'])

        primitive = compute.obj_to_primitive(target_version='1.14')
        self.assertIn('cpu_allocation_ratio', primitive['nova_object.data'])
        self.assertIn('ram_allocation_ratio', primitive['nova_object.data'])

    def test_compat_disk_allocation_ratio(self):
        compute = compute_node.ComputeNode(disk_allocation_ratio=1.0)
        primitive = compute.obj_to_primitive(target_version='1.15')
        self.assertNotIn(
            'disk_allocation_ratio', primitive['nova_object.data'])

        primitive = compute.obj_to_primitive(target_version='1.16')
        self.assertIn('disk_allocation_ratio', primitive['nova_object.data'])

    @mock.patch('nova.db.main.api.compute_node_update')
    def test_compat_allocation_ratios_old_compute(self, mock_update):
        """Tests the scenario that allocation ratios are overridden in config
        and the legacy compute node record from the database has None set for
        the allocation ratio values. The result is that the migrated record
        allocation ratios should reflect the config overrides.
        """
        self.flags(cpu_allocation_ratio=2.0, ram_allocation_ratio=3.0,
                   disk_allocation_ratio=0.9)
        compute_dict = fake_compute_node.copy()
        # old computes don't provide allocation ratios to the table
        compute_dict['cpu_allocation_ratio'] = None
        compute_dict['ram_allocation_ratio'] = None
        compute_dict['disk_allocation_ratio'] = None
        cls = objects.ComputeNode
        compute = cls._from_db_object(self.context, cls(), compute_dict)

        self.assertEqual(2.0, compute.cpu_allocation_ratio)
        self.assertEqual(3.0, compute.ram_allocation_ratio)
        self.assertEqual(0.9, compute.disk_allocation_ratio)

        mock_update.assert_called_once_with(
            self.context, 123, {'cpu_allocation_ratio': 2.0,
                                'ram_allocation_ratio': 3.0,
                                'disk_allocation_ratio': 0.9})

    @mock.patch('nova.db.main.api.compute_node_update')
    def test_compat_allocation_ratios_zero_conf(self, mock_update):
        """Tests that the override allocation ratios are set to 0.0 for
        whatever reason (maybe an old nova.conf sample file is being used)
        and the legacy compute node record has None for allocation ratios,
        so the resulting data migration makes the record allocation ratios
        use the CONF.initial_*_allocation_ratio values.
        """
        self.flags(cpu_allocation_ratio=0.0, ram_allocation_ratio=0.0,
                   disk_allocation_ratio=0.0)
        compute_dict = fake_compute_node.copy()
        # the computes provide allocation ratios None
        compute_dict['cpu_allocation_ratio'] = None
        compute_dict['ram_allocation_ratio'] = None
        compute_dict['disk_allocation_ratio'] = None
        cls = objects.ComputeNode
        compute = cls._from_db_object(self.context, cls(), compute_dict)

        self.assertEqual(
            CONF.initial_cpu_allocation_ratio, compute.cpu_allocation_ratio)
        self.assertEqual(
            CONF.initial_ram_allocation_ratio, compute.ram_allocation_ratio)
        self.assertEqual(
            CONF.initial_disk_allocation_ratio, compute.disk_allocation_ratio)

        mock_update.assert_called_once_with(
            self.context, 123, {'cpu_allocation_ratio': 4.0,
                                'ram_allocation_ratio': 1.0,
                                'disk_allocation_ratio': 1.0})

    @mock.patch('nova.db.main.api.compute_node_update')
    def test_compat_allocation_ratios_None_conf_zero_values(self, mock_update):
        """Tests the scenario that the CONF.*_allocation_ratio overrides are
        left to the default (None) and the compute node record allocation
        ratio values in the DB are 0.0, so they will be migrated to the
        CONF.initial_*_allocation_ratio values.
        """
        # the CONF.x_allocation_ratio is None by default
        compute_dict = fake_compute_node.copy()
        # the computes provide allocation ratios 0.0
        compute_dict['cpu_allocation_ratio'] = 0.0
        compute_dict['ram_allocation_ratio'] = 0.0
        compute_dict['disk_allocation_ratio'] = 0.0
        cls = objects.ComputeNode
        compute = cls._from_db_object(self.context, cls(), compute_dict)

        self.assertEqual(
            CONF.initial_cpu_allocation_ratio, compute.cpu_allocation_ratio)
        self.assertEqual(
            CONF.initial_ram_allocation_ratio, compute.ram_allocation_ratio)
        self.assertEqual(
            CONF.initial_disk_allocation_ratio, compute.disk_allocation_ratio)

        mock_update.assert_called_once_with(
            self.context, 123, {'cpu_allocation_ratio': 4.0,
                                'ram_allocation_ratio': 1.0,
                                'disk_allocation_ratio': 1.0})

    @mock.patch('nova.db.main.api.compute_node_update')
    def test_compat_allocation_ratios_None_conf_None_values(self, mock_update):
        """Tests the scenario that the override CONF.*_allocation_ratio options
        are the default values (None), the compute node record from the DB has
        None values for allocation ratios, so the resulting migrated record
        will have the CONF.initial_*_allocation_ratio values.
        """
        # the CONF.x_allocation_ratio is None by default
        compute_dict = fake_compute_node.copy()
        # # the computes provide allocation ratios None
        compute_dict['cpu_allocation_ratio'] = None
        compute_dict['ram_allocation_ratio'] = None
        compute_dict['disk_allocation_ratio'] = None
        cls = objects.ComputeNode
        compute = cls._from_db_object(self.context, cls(), compute_dict)

        self.assertEqual(
            CONF.initial_cpu_allocation_ratio, compute.cpu_allocation_ratio)
        self.assertEqual(
            CONF.initial_ram_allocation_ratio, compute.ram_allocation_ratio)
        self.assertEqual(
            CONF.initial_disk_allocation_ratio, compute.disk_allocation_ratio)

        mock_update.assert_called_once_with(
            self.context, 123, {'cpu_allocation_ratio': 4.0,
                                'ram_allocation_ratio': 1.0,
                                'disk_allocation_ratio': 1.0})

    def test_get_all_by_not_mapped(self):
        for mapped in (1, 0, 1, 3):
            compute = fake_compute_with_resources.obj_clone()
            compute._context = self.context
            compute.mapped = mapped
            compute.create()
        nodes = compute_node.ComputeNodeList.get_all_by_not_mapped(
            self.context, 2)
        self.assertEqual(3, len(nodes))
        self.assertEqual([0, 1, 1], sorted([x.mapped for x in nodes]))


class TestComputeNodeObject(test_objects._LocalTest,
                            _TestComputeNodeObject):
    pass


class TestRemoteComputeNodeObject(test_objects._RemoteTest,
                                  _TestComputeNodeObject):
    pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils

from nova import context
from nova import exception
from nova import objects
from nova.objects import cell_mapping
from nova import test
from nova.tests import fixtures


SAMPLE_MAPPING = {'uuid': '',
                  'name': 'fake-cell',
                  'transport_url': 'rabbit:///',
                  'database_connection': 'mysql+pymysql:///'}


def create_mapping(**kwargs):
    args = SAMPLE_MAPPING.copy()
    if 'uuid' not in kwargs:
        args['uuid'] = uuidutils.generate_uuid()
    args.update(kwargs)
    ctxt = context.RequestContext()
    return cell_mapping.CellMapping._create_in_db(ctxt, args)


class CellMappingTestCase(test.NoDBTestCase):
    USES_DB_SELF = True

    def setUp(self):
        super(CellMappingTestCase, self).setUp()
        self.useFixture(fixtures.Database(database='api'))
        self.context = context.RequestContext('fake-user', 'fake-project')
        self.mapping_obj = cell_mapping.CellMapping()

    def test_get_by_uuid(self):
        mapping = create_mapping()
        db_mapping = self.mapping_obj._get_by_uuid_from_db(self.context,
                mapping['uuid'])
        for key in self.mapping_obj.fields.keys():
            self.assertEqual(db_mapping[key], mapping[key])

    def test_get_by_uuid_not_found(self):
        self.assertRaises(exception.CellMappingNotFound,
                self.mapping_obj._get_by_uuid_from_db, self.context,
                          uuidutils.generate_uuid())

    def test_save_in_db(self):
        mapping = create_mapping()
        self.mapping_obj._save_in_db(self.context, mapping['uuid'],
                {'name': 'meow'})
        db_mapping = self.mapping_obj._get_by_uuid_from_db(self.context,
                mapping['uuid'])
        self.assertNotEqual(db_mapping['name'], mapping['name'])
        for key in [key for key in self.mapping_obj.fields.keys()
                    if key not in ['name', 'updated_at']]:
            self.assertEqual(db_mapping[key], mapping[key])

    def test_destroy_in_db(self):
        mapping = create_mapping()
        self.mapping_obj._get_by_uuid_from_db(self.context, mapping['uuid'])
        self.mapping_obj._destroy_in_db(self.context, mapping['uuid'])
        self.assertRaises(exception.CellMappingNotFound,
                self.mapping_obj._get_by_uuid_from_db, self.context,
                mapping['uuid'])

    def test_destroy_in_db_not_found(self):
        self.assertRaises(exception.CellMappingNotFound,
                self.mapping_obj._destroy_in_db, self.context,
                uuidutils.generate_uuid())


class CellMappingListTestCase(test.NoDBTestCase):
    USES_DB_SELF = True

    def setUp(self):
        super(CellMappingListTestCase, self).setUp()
        self.useFixture(fixtures.Database(database='api'))

    def test_get_all(self):
        mappings = {}
        mapping = create_mapping()
        mappings[mapping['uuid']] = mapping
        mapping = create_mapping()
        mappings[mapping['uuid']] = mapping

        ctxt = context.RequestContext()
        db_mappings = cell_mapping.CellMappingList._get_all_from_db(ctxt)
        for db_mapping in db_mappings:
            mapping = mappings[db_mapping.uuid]
            for key in cell_mapping.CellMapping.fields.keys():
                self.assertEqual(db_mapping[key], mapping[key])

    def test_get_by_disabled(self):
        enabled_mapping = create_mapping(disabled=False)
        disabled_mapping = create_mapping(disabled=True)

        ctxt = context.RequestContext()
        mappings = cell_mapping.CellMappingList.get_all(ctxt)
        self.assertEqual(2, len(mappings))
        self.assertEqual(enabled_mapping['uuid'], mappings[0].uuid)
        self.assertEqual(disabled_mapping['uuid'], mappings[1].uuid)
        mappings = cell_mapping.CellMappingList.get_by_disabled(ctxt,
                                                                disabled=False)
        self.assertEqual(1, len(mappings))
        self.assertEqual(enabled_mapping['uuid'], mappings[0].uuid)
        mappings = cell_mapping.CellMappingList.get_by_disabled(ctxt,
                                                                disabled=True)
        self.assertEqual(1, len(mappings))
        self.assertEqual(disabled_mapping['uuid'], mappings[0].uuid)

    def test_get_by_project_id(self):
        ctxt = context.RequestContext()
        cell1 = objects.CellMapping.get_by_uuid(ctxt, create_mapping().uuid)
        cell2 = objects.CellMapping.get_by_uuid(ctxt, create_mapping().uuid)
        cell3 = objects.CellMapping.get_by_uuid(ctxt, create_mapping().uuid)
        cells = [cell1, cell2, cell3]

        # Proj1 is all in one cell
        for i in range(0, 5):
            uuid = uuidutils.generate_uuid()
            im = objects.InstanceMapping(context=ctxt,
                                         instance_uuid=uuid,
                                         cell_mapping=cell1,
                                         project_id='proj1')
            im.create()

        # Proj2 is in the first two cells
        for i in range(0, 5):
            uuid = uuidutils.generate_uuid()
            cell = cells[i % 2]
            im = objects.InstanceMapping(context=ctxt,
                                         instance_uuid=uuid,
                                         cell_mapping=cell,
                                         project_id='proj2')
            im.create()

        # One mapping has no cell. This helps ensure that our query
        # filters out any mappings that aren't tied to a cell.
        im = objects.InstanceMapping(context=ctxt,
                                     instance_uuid=uuidutils.generate_uuid(),
                                     cell_mapping=None,
                                     project_id='proj2')
        im.create()

        # Proj1 should only be in cell1 and we should only get back
        # a single mapping for it
        cells = objects.CellMappingList.get_by_project_id(ctxt, 'proj1')
        self.assertEqual(1, len(cells))
        self.assertEqual(cell1.uuid, cells[0].uuid)

        cells = objects.CellMappingList.get_by_project_id(ctxt, 'proj2')
        self.assertEqual(2, len(cells))
        self.assertEqual(sorted([cell1.uuid, cell2.uuid]),
                         sorted([cm.uuid for cm in cells]))



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc
import os

from nova import utils
from nova.virt.libvirt.volume import mount
from nova.virt.libvirt.volume import volume as libvirt_volume


class LibvirtBaseFileSystemVolumeDriver(
    libvirt_volume.LibvirtBaseVolumeDriver, metaclass=abc.ABCMeta):
    """The base class for file system type volume drivers"""

    def __init__(self, host):
        super(LibvirtBaseFileSystemVolumeDriver,
              self).__init__(host, is_block_dev=False)

    @abc.abstractmethod
    def _get_mount_point_base(self):
        """Return the mount point path prefix.

        This is used to build the device path.

        :returns: The mount point path prefix.
        """
        raise NotImplementedError('_get_mount_point_base')

    def _normalize_export(self, export):
        """Normalize the export (share) if necessary.

        Subclasses should override this method if they have a non-standard
        export value, e.g. if the export is a URL. By default this method just
        returns the export value passed in unchanged.

        :param export: The export (share) value to normalize.
        :returns: The normalized export value.
        """
        return export

    def _get_mount_path(self, connection_info):
        """Returns the mount path prefix using the mount point base and share.

        :param connection_info: dict of the form

        ::

          connection_info = {
              'data': {
                  'export': the file system share,
                  ...
              }
              ...
          }

        :returns: The mount path prefix.
        """
        share = self._normalize_export(connection_info['data']['export'])
        return os.path.join(self._get_mount_point_base(),
                            utils.get_hash_str(share))

    def _get_device_path(self, connection_info):
        """Returns the hashed path to the device.

        :param connection_info: dict of the form

        ::

          connection_info = {
              'data': {
                  'export': the file system share,
                  'name': the name of the device,
                  ...
              }
              ...
          }

        :returns: The full path to the device.
        """
        mount_path = self._get_mount_path(connection_info)
        return os.path.join(mount_path, connection_info['data']['name'])


class LibvirtMountedFileSystemVolumeDriver(LibvirtBaseFileSystemVolumeDriver,
                                           metaclass=abc.ABCMeta):
    # NOTE(mdbooth): Hopefully we'll get to the point where everything which
    # previously subclassed LibvirtBaseFileSystemVolumeDriver now subclasses
    # LibvirtMountedFileSystemVolumeDriver. If we get there, we should fold
    # this class into the base class.
    def __init__(self, host, fstype):
        super(LibvirtMountedFileSystemVolumeDriver, self).__init__(host)

        self.fstype = fstype

    def connect_volume(self, connection_info, instance):
        """Connect the volume."""
        export = connection_info['data']['export']
        vol_name = connection_info['data']['name']
        mountpoint = self._get_mount_path(connection_info)

        mount.mount(self.fstype, export, vol_name, mountpoint, instance,
                    self._mount_options(connection_info))

        connection_info['data']['device_path'] = \
            self._get_device_path(connection_info)

    def disconnect_volume(self, connection_info, instance, force=False):
        """Disconnect the volume."""
        vol_name = connection_info['data']['name']
        mountpoint = self._get_mount_path(connection_info)

        mount.umount(vol_name, mountpoint, instance)

    @abc.abstractmethod
    def _mount_options(self, connection_info):
        """Return a list of additional arguments to pass to the mount command.
        """
        pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from webob import exc

from nova.api.openstack.compute import server_tags
from nova.api.openstack.compute import servers
from nova.compute import vm_states
from nova import context
from nova.db.main import models
from nova import exception
from nova import objects
from nova.objects import instance
from nova.objects import tag as tag_obj
from nova import test
from nova.tests.unit.api.openstack import fakes
from nova.tests.unit import fake_instance

UUID = 'b48316c5-71e8-45e4-9884-6c78055b9b13'
TAG1 = 'tag1'
TAG2 = 'tag2'
TAG3 = 'tag3'
TAGS = [TAG1, TAG2, TAG3]
NON_EXISTING_UUID = '123'


def return_server(compute_api, context, instance_id, expected_attrs=None):
    return fake_instance.fake_instance_obj(context, vm_state=vm_states.ACTIVE)


def return_invalid_server(compute_api, context, instance_id,
                          expected_attrs=None):
    return fake_instance.fake_instance_obj(context,
                                           vm_state=vm_states.BUILDING)


class ServerTagsTest(test.TestCase):
    api_version = '2.26'

    def setUp(self):
        super(ServerTagsTest, self).setUp()
        self.controller = server_tags.ServerTagsController()
        inst_map = objects.InstanceMapping(
            project_id=fakes.FAKE_PROJECT_ID,
            cell_mapping=objects.CellMappingList.get_all(
                context.get_admin_context())[1])
        self.stub_out('nova.objects.InstanceMapping.get_by_instance_uuid',
                      lambda s, c, u: inst_map)

    def _get_tag(self, tag_name):
        tag = models.Tag()
        tag.tag = tag_name
        tag.resource_id = UUID
        return tag

    def _get_request(self, url, method):
        request = fakes.HTTPRequest.blank(url, version=self.api_version)
        request.method = method
        return request

    @mock.patch('nova.db.main.api.instance_tag_exists')
    def test_show(self, mock_exists):
        mock_exists.return_value = True
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG1), 'GET')

        self.controller.show(req, UUID, TAG1)
        mock_exists.assert_called_once_with(mock.ANY, UUID, TAG1)

    @mock.patch('nova.db.main.api.instance_tag_get_by_instance_uuid')
    def test_index(self, mock_db_get_inst_tags):
        fake_tags = [self._get_tag(tag) for tag in TAGS]
        mock_db_get_inst_tags.return_value = fake_tags

        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'GET')

        res = self.controller.index(req, UUID)
        self.assertEqual(TAGS, res.get('tags'))
        mock_db_get_inst_tags.assert_called_once_with(mock.ANY, UUID)

    @mock.patch('nova.notifications.base.send_instance_update_notification')
    @mock.patch('nova.db.main.api.instance_tag_set')
    def test_update_all(self, mock_db_set_inst_tags, mock_notify):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        fake_tags = [self._get_tag(tag) for tag in TAGS]
        mock_db_set_inst_tags.return_value = fake_tags
        req = self._get_request(
            '/v2/%s/servers/%s/tags' % (fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        res = self.controller.update_all(req, UUID, body={'tags': TAGS})

        self.assertEqual(TAGS, res['tags'])
        mock_db_set_inst_tags.assert_called_once_with(mock.ANY, UUID, TAGS)
        self.assertEqual(1, mock_notify.call_count)

    def test_update_all_too_many_tags(self):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        fake_tags = {'tags': [str(i) for i in range(
            instance.MAX_TAG_COUNT + 1)]}

        req = self._get_request(
            '/v2/%s/servers/%s/tags' % (fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        self.assertRaises(exception.ValidationError,
                          self.controller.update_all,
                          req, UUID, body=fake_tags)

    def test_update_all_forbidden_characters(self):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        for tag in ['tag,1', 'tag/1']:
            self.assertRaises(exception.ValidationError,
                              self.controller.update_all,
                              req, UUID, body={'tags': [tag, 'tag2']})

    def test_update_all_invalid_tag_type(self):
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        self.assertRaises(exception.ValidationError,
                          self.controller.update_all,
                          req, UUID, body={'tags': [1]})

    def test_update_all_tags_with_one_tag_empty_string(self):
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        self.assertRaises(exception.ValidationError,
                          self.controller.update_all,
                          req, UUID, body={'tags': ['tag1', '']})

    def test_update_all_too_long_tag(self):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        tag = "a" * (tag_obj.MAX_TAG_LENGTH + 1)
        self.assertRaises(exception.ValidationError,
                          self.controller.update_all,
                          req, UUID, body={'tags': [tag]})

    def test_update_all_invalid_tag_list_type(self):
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        self.assertRaises(exception.ValidationError,
                          self.controller.update_all,
                          req, UUID, body={'tags': {'tag': 'tag'}})

    def test_update_all_invalid_instance_state(self):
        self.stub_out('nova.api.openstack.common.get_instance',
                      return_invalid_server)
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'PUT')
        self.assertRaises(exc.HTTPConflict, self.controller.update_all,
                          req, UUID, body={'tags': TAGS})

    @mock.patch('nova.db.main.api.instance_tag_exists')
    def test_show_non_existing_tag(self, mock_exists):
        mock_exists.return_value = False
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG1), 'GET')
        self.assertRaises(exc.HTTPNotFound, self.controller.show,
                          req, UUID, TAG1)

    @mock.patch('nova.notifications.base.send_instance_update_notification')
    @mock.patch('nova.db.main.api.instance_tag_add')
    @mock.patch('nova.db.main.api.instance_tag_get_by_instance_uuid')
    def test_update(self, mock_db_get_inst_tags, mock_db_add_inst_tags,
                    mock_notify):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        mock_db_get_inst_tags.return_value = [self._get_tag(TAG1)]
        mock_db_add_inst_tags.return_value = self._get_tag(TAG2)

        url = '/v2/%s/servers/%s/tags/%s' % (fakes.FAKE_PROJECT_ID, UUID, TAG2)
        location = 'http://localhost' + url
        req = self._get_request(url, 'PUT')
        res = self.controller.update(req, UUID, TAG2, body=None)

        self.assertEqual(201, res.status_int)
        self.assertEqual(0, len(res.body))
        self.assertEqual(location, res.headers['Location'])
        mock_db_add_inst_tags.assert_called_once_with(mock.ANY, UUID, TAG2)
        self.assertEqual(2, mock_db_get_inst_tags.call_count)
        self.assertEqual(1, mock_notify.call_count)

    @mock.patch('nova.db.main.api.instance_tag_get_by_instance_uuid')
    def test_update_existing_tag(self, mock_db_get_inst_tags):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        mock_db_get_inst_tags.return_value = [self._get_tag(TAG1)]

        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG1), 'PUT')
        res = self.controller.update(req, UUID, TAG1, body=None)

        self.assertEqual(204, res.status_int)
        self.assertEqual(0, len(res.body))
        mock_db_get_inst_tags.assert_called_once_with(mock.ANY, UUID)

    @mock.patch('nova.db.main.api.instance_tag_get_by_instance_uuid')
    def test_update_tag_limit_exceed(self, mock_db_get_inst_tags):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        fake_tags = [self._get_tag(str(i))
                     for i in range(instance.MAX_TAG_COUNT)]
        mock_db_get_inst_tags.return_value = fake_tags

        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG2), 'PUT')
        self.assertRaises(exc.HTTPBadRequest, self.controller.update,
                          req, UUID, TAG2, body=None)

    @mock.patch('nova.db.main.api.instance_tag_get_by_instance_uuid')
    def test_update_too_long_tag(self, mock_db_get_inst_tags):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        mock_db_get_inst_tags.return_value = []

        tag = "a" * (tag_obj.MAX_TAG_LENGTH + 1)
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, tag), 'PUT')
        self.assertRaises(exc.HTTPBadRequest, self.controller.update,
                          req, UUID, tag, body=None)

    @mock.patch('nova.db.main.api.instance_tag_get_by_instance_uuid')
    def test_update_forbidden_characters(self, mock_db_get_inst_tags):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        mock_db_get_inst_tags.return_value = []
        for tag in ['tag,1', 'tag/1']:
            req = self._get_request(
                '/v2/%s/servers/%s/tags/%s' % (
                    fakes.FAKE_PROJECT_ID, UUID, tag), 'PUT')
            self.assertRaises(exc.HTTPBadRequest, self.controller.update,
                              req, UUID, tag, body=None)

    def test_update_invalid_instance_state(self):
        self.stub_out('nova.api.openstack.common.get_instance',
                      return_invalid_server)
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG1), 'PUT')
        self.assertRaises(exc.HTTPConflict, self.controller.update, req, UUID,
                          TAG1, body=None)

    @mock.patch('nova.db.main.api.instance_tag_get_by_instance_uuid')
    @mock.patch('nova.notifications.base.send_instance_update_notification')
    @mock.patch('nova.db.main.api.instance_tag_delete')
    def test_delete(self, mock_db_delete_inst_tags, mock_notify,
                    mock_db_get_inst_tags):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG2), 'DELETE')
        self.controller.delete(req, UUID, TAG2)
        mock_db_delete_inst_tags.assert_called_once_with(mock.ANY, UUID, TAG2)
        mock_db_get_inst_tags.assert_called_once_with(mock.ANY, UUID)
        self.assertEqual(1, mock_notify.call_count)

    @mock.patch('nova.db.main.api.instance_tag_delete')
    def test_delete_non_existing_tag(self, mock_db_delete_inst_tags):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)

        def fake_db_delete_tag(context, instance_uuid, tag):
            self.assertEqual(UUID, instance_uuid)
            self.assertEqual(TAG1, tag)
            raise exception.InstanceTagNotFound(instance_id=instance_uuid,
                                                tag=tag)
        mock_db_delete_inst_tags.side_effect = fake_db_delete_tag
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG1), 'DELETE')
        self.assertRaises(exc.HTTPNotFound, self.controller.delete,
                          req, UUID, TAG1)

    def test_delete_invalid_instance_state(self):
        self.stub_out('nova.api.openstack.common.get_instance',
                      return_invalid_server)
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, UUID, TAG2), 'DELETE')
        self.assertRaises(exc.HTTPConflict, self.controller.delete, req, UUID,
                          TAG1)

    @mock.patch('nova.notifications.base.send_instance_update_notification')
    @mock.patch('nova.db.main.api.instance_tag_delete_all')
    def test_delete_all(self, mock_db_delete_inst_tags, mock_notify):
        self.stub_out('nova.api.openstack.common.get_instance', return_server)
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'DELETE')
        self.controller.delete_all(req, UUID)
        mock_db_delete_inst_tags.assert_called_once_with(mock.ANY, UUID)
        self.assertEqual(1, mock_notify.call_count)

    def test_delete_all_invalid_instance_state(self):
        self.stub_out('nova.api.openstack.common.get_instance',
                      return_invalid_server)
        req = self._get_request('/v2/%s/servers/%s/tags' % (
                                    fakes.FAKE_PROJECT_ID, UUID), 'DELETE')
        self.assertRaises(exc.HTTPConflict, self.controller.delete_all, req,
                          UUID)

    def test_show_non_existing_instance(self):
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, NON_EXISTING_UUID, TAG1), 'GET')
        self.assertRaises(exc.HTTPNotFound, self.controller.show, req,
                          NON_EXISTING_UUID, TAG1)

    def test_show_with_details_information_non_existing_instance(self):
        req = self._get_request(
            '/v2/%s/servers/%s' % (
                fakes.FAKE_PROJECT_ID, NON_EXISTING_UUID), 'GET')
        servers_controller = servers.ServersController()
        self.assertRaises(exc.HTTPNotFound, servers_controller.show, req,
                          NON_EXISTING_UUID)

    def test_index_non_existing_instance(self):
        req = self._get_request(
            'v2/%s/servers/%s/tags' % (
                fakes.FAKE_PROJECT_ID, NON_EXISTING_UUID), 'GET')
        self.assertRaises(exc.HTTPNotFound, self.controller.index, req,
                          NON_EXISTING_UUID)

    def test_update_non_existing_instance(self):
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, NON_EXISTING_UUID, TAG1), 'PUT')
        self.assertRaises(exc.HTTPNotFound, self.controller.update, req,
                          NON_EXISTING_UUID, TAG1, body=None)

    def test_update_all_non_existing_instance(self):
        req = self._get_request(
            '/v2/%s/servers/%s/tags' % (
                fakes.FAKE_PROJECT_ID, NON_EXISTING_UUID), 'PUT')
        self.assertRaises(exc.HTTPNotFound, self.controller.update_all, req,
                          NON_EXISTING_UUID, body={'tags': TAGS})

    def test_delete_non_existing_instance(self):
        req = self._get_request(
            '/v2/%s/servers/%s/tags/%s' % (
                fakes.FAKE_PROJECT_ID, NON_EXISTING_UUID, TAG1), 'DELETE')
        self.assertRaises(exc.HTTPNotFound, self.controller.delete, req,
                          NON_EXISTING_UUID, TAG1)

    def test_delete_all_non_existing_instance(self):
        req = self._get_request(
            '/v2/%s/servers/%s/tags' % (
                fakes.FAKE_PROJECT_ID, NON_EXISTING_UUID), 'DELETE')
        self.assertRaises(exc.HTTPNotFound, self.controller.delete_all,
                          req, NON_EXISTING_UUID)
